/*
// jQuery multiSelect
//
// Version 1.0 beta
//
// Cory S.N. LaViska
// A Beautiful Site (http://abeautifulsite.net/)
// 06 April 2008
//
// Visit http://abeautifulsite.net/notebook.php?article=62 for more information
//
// Usage: jQuery('#control_id').multiSelect( options, callback )
//
// Options:  selectAll          - whether or not to display the Select All option; true/false, default = true
//           selectAllText      - text to display for selecting/unselecting all options simultaneously
//           noneSelected       - text to display when there are no selected items in the list
//           oneOrMoreSelected  - text to display when there are one or more selected items in the list
//                                (note: you can use % as a placeholder for the number of items selected).
//                                Use * to show a comma separated list of all selected; default = '% selected'
//
// Dependencies:  jQuery 1.2 or higher (http://jquery.com/)
//                the jQuery Dimensions plugin (http://plugins.jquery.com/project/dimensions)
//
// Licensing & Terms of Use
// 
// jQuery File Tree is licensed under a Creative Commons License and is copyrighted (C)2008 by Cory S.N. LaViska.
// For details, visit http://creativecommons.org/licenses/by/3.0/us/
//	
*/
if(jQuery) (function(jQuery){
	
	jQuery.extend(jQuery.fn, {
		multiSelect: function(o, callback) {
			// Default options
			if( !o ) var o = {};
			if( o.selectAll == undefined ) o.selectAll = true;
			if( o.selectAllText == undefined ) o.selectAllText = "اختيار الكل";
			if( o.noneSelected == undefined ) o.noneSelected = '   ---اختر---';
			if( o.oneOrMoreSelected == undefined ) o.oneOrMoreSelected = '% selected';
			
			// Initialize each multiSelect
			jQuery(this).each( function() {
				var select = jQuery(this);
				var html = '<input type="text" readonly="readonly" class="multiSelect" value="" style="cursor: default;" />';
				html += '<div class="multiSelectOptions" style="position: absolute; z-index: 99999; display: none;">';
				if( o.selectAll ) html += '<label class="selectAll"><input type="checkbox" class="selectAll" />' + o.selectAllText + '</label>';
				jQuery(select).find('OPTION').each( function() {
					if( jQuery(this).val() != '' ) {
						html += '<label><input type="checkbox" name="' + jQuery(select).attr('name') + '" value="' + jQuery(this).val() + '"';
						if( jQuery(this).attr('selected') ) html += ' checked="checked"';
                        //alert(jQuery(this).attr('selected'));
						html += ' />' + jQuery(this).html() + '</label>';
					}
				});
				html += '</div>';
				jQuery(select).after(html);
				
				// Events
				jQuery(select).next('.multiSelect').mouseover( function() {
					jQuery(this).addClass('hover');
				}).mouseout( function() {
					jQuery(this).removeClass('hover');
				}).click( function() {
					// Show/hide on click
					if( jQuery(this).hasClass('active') ) {
						jQuery(this).multiSelectOptionsHide();
					} else {
						jQuery(this).multiSelectOptionsShow();
					}
					return false;
				}).focus( function() {
					// So it can be styled with CSS
					jQuery(this).addClass('focus');
				}).blur( function() {
					// So it can be styled with CSS
					jQuery(this).removeClass('focus');
				});
				
				// Determine if Select All should be checked initially
				if( o.selectAll ) {
					var sa = true;
					jQuery(select).next('.multiSelect').next('.multiSelectOptions').find('INPUT:checkbox').not('.selectAll').each( function() {
						if( !jQuery(this).attr('checked') ) sa = false;
					});
					if( sa ) jQuery(select).next('.multiSelect').next('.multiSelectOptions').find('INPUT.selectAll').attr('checked', true).parent().addClass('checked');
				}
				
				// Handle Select All
				jQuery(select).next('.multiSelect').next('.multiSelectOptions').find('INPUT.selectAll').click( function() {
					if( jQuery(this).attr('checked') == true ) jQuery(this).parent().parent().find('INPUT:checkbox').attr('checked', true).parent().addClass('checked'); else jQuery(this).parent().parent().find('INPUT:checkbox').attr('checked', false).parent().removeClass('checked');
				});
				
				// Handle checkboxes
				jQuery(select).next('.multiSelect').next('.multiSelectOptions').find('INPUT:checkbox').click( function() {
					jQuery(this).parent().parent().multiSelectUpdateSelected(o);
					jQuery(this).parent().parent().find('LABEL').removeClass('checked').find('INPUT:checked').parent().addClass('checked');
					jQuery(this).parent().parent().prev('.multiSelect').focus();
					if( !jQuery(this).attr('checked') ) jQuery(this).parent().parent().find('INPUT:checkbox.selectAll').attr('checked', false).parent().removeClass('checked');
					if( callback ) callback(jQuery(this));
				});
				
				// Initial display
				jQuery(select).next('.multiSelect').next('.multiSelectOptions').each( function() {
					jQuery(this).multiSelectUpdateSelected(o);
					jQuery(this).find('INPUT:checked').parent().addClass('checked');
				});
				
				// Handle hovers
				jQuery(select).next('.multiSelect').next('.multiSelectOptions').find('LABEL').mouseover( function() {
					jQuery(this).parent().find('LABEL').removeClass('hover');
					jQuery(this).addClass('hover');
				}).mouseout( function() {
					jQuery(this).parent().find('LABEL').removeClass('hover');
				});
				
				// Keyboard
				jQuery(select).next('.multiSelect').keydown( function(e) {
					// Is dropdown visible?
					if( jQuery(this).next('.multiSelectOptions').is(':visible') ) {
						// Dropdown is visible
						// Tab
						if( e.keyCode == 9 ) {
							jQuery(this).addClass('focus').trigger('click'); // esc, left, right - hide
							jQuery(this).focus().next(':input').focus();
							return true;
						}
						
						// ESC, Left, Right
						if( e.keyCode == 27 || e.keyCode == 37 || e.keyCode == 39 ) {
							// Hide dropdown
							jQuery(this).addClass('focus').trigger('click');
						}
						// Down
						if( e.keyCode == 40 ) {
							if( !jQuery(this).next('.multiSelectOptions').find('LABEL').hasClass('hover') ) {
								// Default to first item
								jQuery(this).next('.multiSelectOptions').find('LABEL:first').addClass('hover');
							} else {
								// Move down, cycle to top if on bottom
								jQuery(this).next('.multiSelectOptions').find('LABEL.hover').removeClass('hover').next('LABEL').addClass('hover');
								if( !jQuery(this).next('.multiSelectOptions').find('LABEL').hasClass('hover') ) {
									jQuery(this).next('.multiSelectOptions').find('LABEL:first').addClass('hover');
								}
							}
							return false;
						}
						// Up
						if( e.keyCode == 38 ) {
							if( !jQuery(this).next('.multiSelectOptions').find('LABEL').hasClass('hover') ) {
								// Default to first item
								jQuery(this).next('.multiSelectOptions').find('LABEL:first').addClass('hover');
							} else {
								// Move up, cycle to bottom if on top
								jQuery(this).next('.multiSelectOptions').find('LABEL.hover').removeClass('hover').prev('LABEL').addClass('hover');
								if( !jQuery(this).next('.multiSelectOptions').find('LABEL').hasClass('hover') ) {
									jQuery(this).next('.multiSelectOptions').find('LABEL:last').addClass('hover');
								}
							}
							return false;
						}
						// Enter, Space
						if( e.keyCode == 13 || e.keyCode == 32 ) {
							// Select All
							if( jQuery(this).next('.multiSelectOptions').find('LABEL.hover INPUT:checkbox').hasClass('selectAll') ) {
								if( jQuery(this).next('.multiSelectOptions').find('LABEL.hover INPUT:checkbox').attr('checked') ) {
									// Uncheck all
									jQuery(this).next('.multiSelectOptions').find('INPUT:checkbox').attr('checked', false).parent().removeClass('checked');
								} else {
									// Check all
									jQuery(this).next('.multiSelectOptions').find('INPUT:checkbox').attr('checked', true).parent().addClass('checked');
								}
								jQuery(this).next('.multiSelectOptions').multiSelectUpdateSelected(o);
								if( callback ) callback(jQuery(this));
								return false;
							}
							// Other checkboxes
							if( jQuery(this).next('.multiSelectOptions').find('LABEL.hover INPUT:checkbox').attr('checked') ) {
								// Uncheck
								jQuery(this).next('.multiSelectOptions').find('LABEL.hover INPUT:checkbox').attr('checked', false);
								jQuery(this).next('.multiSelectOptions').multiSelectUpdateSelected(o);
								jQuery(this).next('.multiSelectOptions').find('LABEL').removeClass('checked').find('INPUT:checked').parent().addClass('checked');
								// Select all status can't be checked at this point
								jQuery(this).next('.multiSelectOptions').find('INPUT:checkbox.selectAll').attr('checked', false).parent().removeClass('checked');
								if( callback ) callback(jQuery(this));
							} else {
								// Check
								jQuery(this).next('.multiSelectOptions').find('LABEL.hover INPUT:checkbox').attr('checked', true);
								jQuery(this).next('.multiSelectOptions').multiSelectUpdateSelected(o);
								jQuery(this).next('.multiSelectOptions').find('LABEL').removeClass('checked').find('INPUT:checked').parent().addClass('checked');
								if( callback ) callback(jQuery(this));
							}
						}
						return false;
					} else {
						// Dropdown is not visible
						if( e.keyCode == 38 || e.keyCode == 40 || e.keyCode == 13 || e.keyCode == 32 ) { // down, enter, space - show
							// Show dropdown
							jQuery(this).removeClass('focus').trigger('click');
							jQuery(this).next('.multiSelectOptions').find('LABEL:first').addClass('hover');
							return false;
						}
						//  Tab key
						if( e.keyCode == 9 ) {
							// Shift focus to next INPUT element on page
							jQuery(this).focus().next(':input').focus();
							return true;
						}
					}
					// Prevent enter key from submitting form
					if( e.keyCode == 13 ) return false;
				});
				
				// Eliminate the original form element
				jQuery(select).remove();
			});
			
		},
		
		// Hide the dropdown
		multiSelectOptionsHide: function() {
			jQuery(this).removeClass('active').next('.multiSelectOptions').hide();
		},
		
		// Show the dropdown
		multiSelectOptionsShow: function() {
			// Hide any open option boxes
			jQuery('.multiSelect').multiSelectOptionsHide();
			jQuery(this).next('.multiSelectOptions').find('LABEL').removeClass('hover');
			jQuery(this).addClass('active').next('.multiSelectOptions').show();
			
			// Position it
			var offset = jQuery(this).offset();
			jQuery(this).next('.multiSelectOptions').css({ top:  offset.top + jQuery(this).outerHeight() + 'px' });
			jQuery(this).next('.multiSelectOptions').css({ left: offset.left + 'px' });
			
			// Disappear on hover out
			multiSelectCurrent = jQuery(this);
			var timer = '';
			jQuery(this).next('.multiSelectOptions').hover( function() {
				clearTimeout(timer);
			}, function() {
				timer = setTimeout('jQuery(multiSelectCurrent).multiSelectOptionsHide(); jQuery(multiSelectCurrent).unbind("hover");', 250);
			});
			
		},
		
		// Update the textbox with the total number of selected items
		multiSelectUpdateSelected: function(o) {
			var i = 0, s = '';
			jQuery(this).find('INPUT:checkbox:checked').not('.selectAll').each( function() {
				i++;
			})
			if( i == 0 ) {
				jQuery(this).prev('INPUT.multiSelect').val( o.noneSelected );
			} else {
				if( o.oneOrMoreSelected == '*' ) {
					var display = '';
					jQuery(this).find('INPUT:checkbox:checked').each( function() {
						if( jQuery(this).parent().text() != o.selectAllText ) display = display + jQuery(this).parent().text() + ', ';
					});
					display = display.substr(0, display.length - 2);
					jQuery(this).prev('INPUT.multiSelect').val( display );
				} else {
					jQuery(this).prev('INPUT.multiSelect').val( o.oneOrMoreSelected.replace('%', i) );
				}
			}
		}
		
	});
	
})(jQuery);