﻿<?php

		/**
		 * class debugger
		 * Gives dal te oppertunity to debug the stuff that is sent to the database, even the things the driver does
		 * and gives you a little debugger for finding errors in your code.
		 *
		 * @author		Patrick van Zweden 
		 * @version		0.1.0
		 * @package extensions
		 * @subpackage debugger
		 */
	class debugger {

		var $dbdriver;
		var $diskwriter;

		
		/**
		 * This is the constructor of the debugger class.
		 * Only thing it does is calling the initialize proc.
		 */
		function debugger() 
		{
			$this->initialize();
		}

		/**
		 * This function initializes the vars that are used.
		 */
		function initialize()
		{
			
			include_once "debugger/class_diskwriter.phps";
			include_once "debugger/config.php";
		
			$this->configuration = new debugConfiguration;

			$this->status = 0;
			$this->queryHook = &$this->configuration->queryHook;
			$this->selectDBHook = &$this->configuration->selectDBHook;
			$this->transactionHook = &$this->configuration->transactionHook;
			$this->final = &$this->configuration->final;
	
			$this->diskwriter = new diskwriter;
		}

		
		/**
		 * This function will debug a query which is going to bes sent to the database.
		 *
		 * @access public
		 * @param (string) $query 
		 * @return boolean
		 */
		function executeQuery($query)
		{

			if (eregi("^select", $query)) {

				if (!$this->dbdriver->executeQuery($query)) {
					$this->debugMessage(sprintf(DEBUG_EXECUTE_QUERY_FAILED, $query, rtrim($this->dbdriver->getDBError())));
					$this->dbdriver->saveState();
					$returnValue = false;
				}
				else {
					$this->debugMessage(sprintf(DEBUG_QUERY_EXECUTED, $query));
					$returnValue = true;
	
					if ($this->configuration->dumpResult) {
						$rowcount = $this->dbdriver->resultRowcount();
						$this->debugMessage(sprintf(DEBUG_RESULT_ROWS, $rowcount));

						if ($rowcount > 0) {
							$resultArray = $this->dbdriver->fetchArray();
							$this->debugMessage(DEBUG_DUMPING_RESULT);

							while (list($key, $value) = each($resultArray)) {
								$fieldlist .= $key . ";";
								$valuelist .= $value . ";";
							}

							$this->debugMessage($fieldlist);
							$this->debugMessage($valuelist);

							while ($resultArray = $this->dbdriver->fetchArray()) {
								$valuelist = "";

								while (list($key, $value) = each($resultArray)) {
									$valuelist .= $value . ";";
								}
								$this->debugMessage($valuelist);
							}
						}
						$returnValue = false;
					}
					$this->dbdriver->saveState();
				}
			}
			else {

				if (!$this->dbdriver->executeQuery($query)) {
					$this->debugMessage(sprintf(DEBUG_EXECUTE_QUERY_FAILED, $query, rtrim($this->dbdriver->getDBError())));
					$this->dbdriver->saveState();
				}
				else {
					$this->debugMessage(sprintf(DEBUG_QUERY_EXECUTED, $query));
					$this->debugMessage(sprintf(DEBUG_AFFECTED, $this->dbdriver->affectedRows()));

					$this->dbdriver->saveState();
					$returnValue = true;
				}
			}

			$this->dbdriver->restoreState();

			return $returnValue;
		}


		/**
		 * This function just ouputs a message to the debugfile.
		 * This can be anything, from things the driver want's to put in the log
		 * to things you want to enter in the log from within your program.
		 *
		 * @access public
		 * @param (string) $message
		 * @return boolean
		 */
		function debugMessage($message)
		{
			$message = sprintf("*** %s *** %s", strftime("%x - %X"), $message);
			$this->diskwriter->writeMessage($message);
		}


		/**
		 * Catches the start transaction function
		 *
		 * @access public
		 */
		function startTransaction()
		{
			$this->debugMessage(DEBUG_TRANS_START);

			return false;
		}

		/**
		 * catches the commit of a transaction
		 *
		 * @access public
		 */
		function commitTransaction()
		{
			$this->debugMessage(DEBUG_TRANS_COMMIT);

			return false;
		}

		/**
		 * Catches the abort of a transaction
		 *
		 * @access public
		 */
		function abortTransaction()
		{
			$this->debugMessage(DEBUG_TRANS_ABORT);

			return false;
		}

		/**
		 * Catches the selecttion of a database
		 *
		 * @access public
		 */
		function selectDatabase($database)
		{
			$this->debugMessage(sprintf(DEBUG_DB_SELECTED, $database));
		}

		/**
		 * Destructor
		 *
		 * @access public
		 */
		function onDestroy()
		{
			$this->diskwriter->onDestroy();
		}


}
?>
